#include <J3ML/Geometry/Circle.hpp>



namespace J3ML
{
    Circle::Circle(const Vector3& center, const Vector3& norm, float radius)
    : Position(center), Normal(norm), Radius(radius)
    {}

    Vector3 Circle::BasisU() const { return Normal.Perpendicular(); }
    Vector3 Circle::BasisV() const { return Normal.AnotherPerpendicular();}


    Vector3 Circle::GetPoint(float angleRadians) const
    {
        float sin, cos;
        Math::SinCos(angleRadians, sin, cos);
        return Position + Radius * (sin * BasisU() + cos * BasisV());
    }

    Vector3 Circle::GetPoint(float angleRadians, float d) const
    {
        float sin, cos;
        Math::SinCos(angleRadians, sin, cos);
        return Position + Radius * d * (sin * BasisU() + cos * BasisV());
    }

    Vector3 Circle::ExtremePoint(const Vector3& direction) const
    {
        Vector3 d = direction - direction.ProjectToNorm(Normal);

        if (d.IsZero())
            return Position;
        else
            return Position + d.ScaledToLength(Radius);
    }
}