
#include <jtest/jtest.hpp>
#include <jtest/Unit.hpp>
#include <J3ML/LinearAlgebra/Vector2.hpp>

jtest::Unit Vector2Unit {"Vector2"};
namespace Vector2Tests
{
    inline void Define()
    {
        using namespace jtest;
        using J3ML::LinearAlgebra::Vector2;

        Vector2Unit += Test("Ctor_Default", []
        {
            // TODO: implement check_eq
            jtest::check(Vector2() == Vector2::Zero);
        });

        Vector2Unit += Test("Ctor_XY", []
        {
            Vector2 vec (1, 0);
            // TODO: implement check_eq
            jtest::check(vec == Vector2::Right);
        });

        Vector2Unit += Test("Addition_Op", []
        {
            Vector2 A (1,1);
            Vector2 B (2,2);

            Vector2 C (3, 3);
            // TODO: implement check_eq
            jtest::check(A+B == C);
        });

        Vector2Unit += Test("Addition_Method", []
        {
            Vector2 A (2,2);
            Vector2 B (2,2);

            Vector2 C (4, 4);
            // TODO: implement check_eq
            jtest::check(A.Add(B) == C);
        });

        Vector2Unit += Test("Addition_Static", []
        {
            Vector2 A (3, 3);
            Vector2 B (2, 2);

            Vector2 C (5, 5);
            // TODO: implement check_eq
            jtest::check(Vector2::Add(A, B) == C);
        });

        Vector2Unit += Test("Subtract_Op", []
        {
            Vector2 A (1,1);
            Vector2 B (2,2);

            Vector2 C (-1, -1);

            jtest::check(A-B == C);
        });

        Vector2Unit += Test("Subtract_Method", []
        {
            Vector2 A (1,1);
            Vector2 B (2,2);

            Vector2 C (-1, -1);

            jtest::check(A.Sub(B) == C);
        });

        Vector2Unit += Test("Subtract_Static", []
        {
            Vector2 A (1,1);
            Vector2 B (2,2);

            Vector2 C (-1, -1);

            jtest::check(Vector2::Sub(A, B) == C);
        });

        Vector2Unit += Test("Scalar_Multiplication", []
        {
            Vector2 A (5, 1);
            float B = 0.5f;
            Vector2 C (2.5f, .5f);
            jtest::check(A*B == C);
        });

        Vector2Unit += Test("Size", []
        {
            jtest::check(sizeof(Vector2) == 8);
        });

        Vector2Unit += Test("NaN", []
        {

            jtest::check(Vector2::Zero != Vector2::NaN);
            jtest::check(Vector2::Up != Vector2::NaN);
            jtest::check(Vector2::Left != Vector2::NaN);
            jtest::check(Vector2::Down != Vector2::NaN);
            jtest::check(Vector2::Right != Vector2::NaN);
            jtest::check(Vector2::NaN != Vector2::NaN);
        });

        Vector2Unit += Test("MarginOfError", []
        {
            Vector2 A (2,2);
            Vector2 B (1.85, 1.85);

            jtest::check(A.IsWithinMarginOfError(B, 0.5f));
        });

        Vector2Unit += Test("Min", []
        {
            Vector2 A (2,2);
            Vector2 B (1.85, 1.85);

            jtest::check( Vector2::Min(A, B) == B);
        });

        Vector2Unit += Test("Max", []
        {
            Vector2 A (2,2);
            Vector2 B (1.85, 1.85);

            jtest::check( Vector2::Max(A, B) == A);
        });

        Vector2Unit += Test("Clamp", []
        {
            Vector2 Input (0, 20);

            Vector2 Minimum ( 2, 2);
            Vector2 Maximum (16, 16);

            Vector2 ExpectedResult (2, 16);

            jtest::check(Input.Clamp(Minimum, Maximum) == ExpectedResult);
        });

        Vector2Unit += Test("DotProduct", []
        {

            Vector2 A (2, 2);
            Vector2 B (1, 1);
            jtest::check(A.Dot(B) == 4.f);
        });

        Vector2Unit += Test("Project", []
        {
            Vector2 Base (4, 4);
            Vector2 Projected (1, 0);

            Vector2 ExpectedResult (4, 0);

            jtest::check(Base.Project(Projected) == ExpectedResult);
        });

        Vector2Unit += Test("Normalized", []
        {
            Vector2 A(2, 0);
            Vector2 B(1, 0);
            jtest::check(A.Normalized() == B);
        });

        Vector2Unit += Test("Lerp", []
        {
            Vector2 A (2,2);
            Vector2 B (10, 10);
            Vector2 C (6, 6);

            jtest::check(A.Lerp(B, 0.f) == A);
            jtest::check(A.Lerp(B, 1.f) == B);
            jtest::check(A.Lerp(B, 0.5f) == C);
        });

        Vector2Unit += Test("AngleBetween", []
        {
            Vector2 A (0.5f, 0.5);
            Vector2 B (0.5f, 0.1f);

            A = A.Normalized();
            B = B.Normalized();

            jtest::check_float_eq(A.AngleBetween(B), 0.58800244);
        });
    }

    inline void Run()
    {
        Vector2Unit.RunAll();
    }
}


